#[non_exhaustive]#[repr(u32)]pub enum ProcessState {
Running = 0,
Locked = 1,
Checkpointed = 2,
Failed = 3,
Unknown(u32),
}Expand description
CUDA process state used by the checkpoint and restore driver APIs.
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Running = 0
The process can make CUDA API calls.
Locked = 1
CUDA API locks are taken and further CUDA API calls will block.
Checkpointed = 2
GPU memory has been moved to host memory and device handles were released.
Failed = 3
The process entered an unrecoverable error during checkpoint or restore.
Unknown(u32)
Unknown process state returned by a newer CUDA driver version.