pub struct HostNodeParams { /* private fields */ }Implementations
Sourceimpl HostNodeParams
impl HostNodeParams
Sourcepub const unsafe fn new(func: HostFunction, user_data: *mut ()) -> Self
pub const unsafe fn new(func: HostFunction, user_data: *mut ()) -> Self
Creates host callback node parameters from a raw user-data pointer.
Safety
user_data must remain valid for func according to CUDA host-node callback rules until no graph execution can invoke the callback.
pub const fn function(self) -> HostFunction
pub const fn user_data(self) -> *mut ()