MemoryCopy3DNodeParams

Struct MemoryCopy3DNodeParams 

pub struct MemoryCopy3DNodeParams { /* private fields */ }

Implementations

Source

impl MemoryCopy3DNodeParams

pub const unsafe fn new( src_array: Option<ArrayHandle>, src_pos: Position, src_ptr: PitchedPtr, dst_array: Option<ArrayHandle>, dst_pos: Position, dst_ptr: PitchedPtr, extent: Extent, kind: MemoryCopyKind, ) -> Self

Creates three-dimensional memcpy node parameters from raw array and pitched pointer operands.

Safety

Every array and pointer operand selected by kind must remain valid for the region described by the positions, pitched pointers, and extent whenever CUDA evaluates a graph node using this value.

pub const fn src_array(self) -> Option<ArrayHandle>

pub const fn src_pos(self) -> Position

pub const fn src_ptr(self) -> PitchedPtr

pub const fn dst_array(self) -> Option<ArrayHandle>

pub const fn dst_pos(self) -> Position

pub const fn dst_ptr(self) -> PitchedPtr

pub const fn extent(self) -> Extent

pub const fn kind(self) -> MemoryCopyKind