pub struct PitchedPtr {
pub pitch: usize,
pub x_size: usize,
pub y_size: usize,
/* private fields */
}Fields
pitch: usizex_size: usizey_size: usizeImplementations
Sourceimpl PitchedPtr
impl PitchedPtr
Sourcepub const unsafe fn new(
ptr: *mut (),
pitch: usize,
x_size: usize,
y_size: usize,
) -> Self
pub const unsafe fn new( ptr: *mut (), pitch: usize, x_size: usize, y_size: usize, ) -> Self
Creates pitched pointer parameters from a raw device or mapped host pointer.
Safety
ptr must be valid for every row described by pitch, x_size, and y_size when CUDA evaluates the graph node using this value.
pub const fn ptr(self) -> *mut ()