pub struct ExecutableGraphLaunchOperation<'graph> { /* private fields */ }Implementations
Sourceimpl ExecutableGraphLaunchOperation<'_>
impl ExecutableGraphLaunchOperation<'_>
Sourcepub fn enqueue(self, stream: &Stream) -> Result<()>
pub fn enqueue(self, stream: &Stream) -> Result<()>
Enqueues this graph launch in stream.
Errors
Returns an error if CUDA rejects the graph operation, if stream belongs to a different context, if a previous asynchronous launch reported an error, or if CUDA reports runtime initialization diagnostics.
pub const fn graph(&self) -> &ExecutableGraph