MemorySetNodeParams

Struct MemorySetNodeParams 

pub struct MemorySetNodeParams { /* private fields */ }

Implementations

Source

impl MemorySetNodeParams

pub const unsafe fn new(dst: DevicePtr, element_size: u32, width: usize) -> Self

Creates raw memset node parameters.

Safety

dst must be valid for writes of element_size * width bytes when the graph executes. If height or pitch are changed after construction, the caller must account for those values as required by CUDA.

pub const fn with_pitch(self, pitch: usize) -> Self

pub const fn with_value(self, value: u32) -> Self

pub const fn with_height(self, height: usize) -> Self

pub const fn dst(self) -> DevicePtr

pub const fn pitch(self) -> usize

pub const fn value(self) -> u32

pub const fn element_size(self) -> u32

pub const fn width(self) -> usize

pub const fn height(self) -> usize