JitOptions

Struct JitOptions 

pub struct JitOptions<'a> {
Show 13 fields pub max_registers: Option<u32>, pub threads_per_block: Option<u32>, pub wall_time: Option<&'a mut f32>, pub info_log_buffer: Option<&'a mut [u8]>, pub error_log_buffer: Option<&'a mut [u8]>, pub optimization_level: Option<u32>, pub target_from_cuda_context: Option<()>, pub target: Option<JitTarget>, pub fallback_strategy: Option<JitFallback>, pub generate_debug_info: Option<bool>, pub log_verbose: Option<bool>, pub generate_line_info: Option<bool>, pub cache_mode: Option<JitCacheMode>,
}

Fields

max_registers: Option<u32>threads_per_block: Option<u32>wall_time: Option<&'a mut f32>info_log_buffer: Option<&'a mut [u8]>error_log_buffer: Option<&'a mut [u8]>optimization_level: Option<u32>target_from_cuda_context: Option<()>target: Option<JitTarget>fallback_strategy: Option<JitFallback>generate_debug_info: Option<bool>log_verbose: Option<bool>generate_line_info: Option<bool>cache_mode: Option<JitCacheMode>

Implementations

Source

impl<'a> JitOptions<'a>

pub fn with_max_registers(self, value: u32) -> Self

pub fn with_threads_per_block(self, value: u32) -> Self

pub fn with_wall_time(self, value: &'a mut f32) -> Self

pub fn with_info_log(self, buffer: &'a mut [u8]) -> Self

pub fn with_error_log(self, buffer: &'a mut [u8]) -> Self

pub fn with_optimization_level(self, level: u32) -> Self

pub const fn with_target_from_cuda_context(self) -> Self

pub fn with_target(self, target: JitTarget) -> Self

pub fn with_fallback_strategy(self, strategy: JitFallback) -> Self

pub fn with_generate_debug_info(self, enable: bool) -> Self

pub fn with_log_verbose(self, enable: bool) -> Self

pub fn with_generate_line_info(self, enable: bool) -> Self

pub fn with_cache_mode(self, mode: JitCacheMode) -> Self

pub fn build(&mut self) -> JitOptionsArtifact

Prepares the option and value arrays for the FFI call. Populates the internal storage fields to keep pointers stable.

Safety

The caller must ensure that this JitOptions instance outlives the FFI call.