MemoryCopyOperation

Struct MemoryCopyOperation 

pub struct MemoryCopyOperation<'a, T> { /* private fields */ }

Implementations

Source

impl<'a, T> MemoryCopyOperation<'a, T>

pub const unsafe fn new( dst: *mut T, src: *const T, count: usize, kind: MemoryCopyKind, ) -> Self

Creates a stream-capture-safe memcpy operation from raw pointers.

Safety

Capturing this operation stores dst and src pointer addresses in the resulting CUDA graph. The caller must ensure both pointers are valid for count elements whenever a captured graph using this operation is launched. If dst is mutable memory, it must also remain exclusive for the work ordered by those launches.