pub trait PushKernelArg {
// Required method
fn push_to<'a>(self, params: &mut KernelParameters<'a>);
}Expand description
Appends a value to a CUDA kernel parameter list.
Implementations convert Rust wrapper types into the value a CUDA kernel sees at the ABI boundary, such as a scalar or device pointer.
Required Methods
fn push_to<'a>(self, params: &mut KernelParameters<'a>)
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".