Program

Struct Program 

pub struct Program { /* private fields */ }

Implementations

Source

impl Program

pub fn create() -> Result<Self>

pub unsafe fn from_raw(handle: nvvmProgram) -> Result<Self>

Takes ownership of a raw NVVM program handle.

Safety

handle must be a valid nvvmProgram that is not owned by any other wrapper. The returned wrapper destroys it with nvvmDestroyProgram.

pub fn add_module(&mut self, module: Module<'_>) -> Result<()>

pub fn lazy_add_module(&mut self, module: Module<'_>) -> Result<()>

pub fn compile(&self, options: &[&str]) -> Result<()>

pub fn compile_with_options(&self, options: &CompileOptions<'_>) -> Result<()>

pub fn verify(&self, options: &[&str]) -> Result<()>

pub fn verify_with_options(&self, options: &CompileOptions<'_>) -> Result<()>

pub fn compiled_result(&self) -> Result<Vec<u8>>

pub fn compiled_image(&self) -> Result<ModuleImage<'static>>

pub fn compiled_string(&self) -> Result<String>

pub fn log(&self) -> Result<String>

pub const fn as_raw(&self) -> nvvmProgram

pub fn into_raw(self) -> nvvmProgram

Transfers ownership of the raw NVVM program handle to the caller.

The caller becomes responsible for destroying it with nvvmDestroyProgram.