Module stream

Module stream 

Structs

BorrowedStream
Stream
StreamCaptureInfo
StreamCaptureScope
StreamFlags
Flags for CUDA stream creation (Context::create_stream_with_flags).
StreamScope

Enums

StreamBinding
StreamCaptureDependencyUpdate
Flags for Stream::update_capture_dependencies_with_dependencies
StreamCaptureMode
StreamCaptureStatus

Traits

GraphRecordable
Operation that may be recorded into a CUDA graph capture scope.

Functions

exchange_capture_mode
Sets the calling thread’s stream capture interaction mode, returning the previous mode for the thread. To facilitate deterministic behavior across function or module boundaries, use this in a push-pop fashion.