pub struct HostFunction(/* private fields */);Expand description
CUDA host callback function pointer.
This is a borrowed function pointer value, not an owned CUDA resource.
Implementations
Sourceimpl HostFunction
impl HostFunction
Sourcepub const unsafe fn new(raw: CUhostFn) -> Self
pub const unsafe fn new(raw: CUhostFn) -> Self
Wraps a raw CUDA host callback function pointer.
Safety
raw must be a callback function pointer with the ABI and lifetime
required by CUDA for every operation that uses the returned handle.
Sourcepub const unsafe fn from_raw(raw: CUhostFn) -> Self
pub const unsafe fn from_raw(raw: CUhostFn) -> Self
Wraps a raw CUDA host callback function pointer.
Safety
raw must be a callback function pointer with the ABI and lifetime
required by CUDA for every operation that uses the returned handle.
pub const fn as_raw(self) -> CUhostFn