DeviceRepr

Trait DeviceRepr 

pub unsafe trait DeviceRepr: Copy + 'static { }
A Rust type that can be represented as plain CUDA device memory.

Safety

Implementors must have a stable bit representation for device memory. Values may be copied byte-for-byte between host and device memory without running Rust destructors or relying on host-only pointer validity.

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementations on Foreign Types

Source

impl DeviceRepr for bool

Source

impl DeviceRepr for f32

Source

impl DeviceRepr for f64

Source

impl DeviceRepr for i8

Source

impl DeviceRepr for i16

Source

impl DeviceRepr for i32

Source

impl DeviceRepr for i64

Source

impl DeviceRepr for i128

Source

impl DeviceRepr for isize

Source

impl DeviceRepr for u8

Source

impl DeviceRepr for u16

Source

impl DeviceRepr for u32

Source

impl DeviceRepr for u64

Source

impl DeviceRepr for u128

Source

impl DeviceRepr for usize

Implementors