pub struct RawExecutableGraph { /* private fields */ }Implementations
Sourceimpl RawExecutableGraph
impl RawExecutableGraph
Sourcepub unsafe fn from_raw(handle: cudaGraphExec_t) -> Result<Self>
pub unsafe fn from_raw(handle: cudaGraphExec_t) -> Result<Self>
Wraps an existing CUDA executable graph handle and takes ownership of it.
Safety
handle must be a valid CUDA executable graph handle.
Ownership of handle is transferred to the returned RawExecutableGraph, and the handle must not be destroyed elsewhere after calling this function.
pub const fn as_raw(&self) -> cudaGraphExec_t
Sourcepub fn into_raw(self) -> cudaGraphExec_t
pub fn into_raw(self) -> cudaGraphExec_t
Consumes the executable graph and returns the raw CUDA executable graph handle without destroying it.
The caller becomes responsible for eventually destroying the returned handle with CUDA.