IpcMemoryHandle

Struct IpcMemoryHandle 

pub struct IpcMemoryHandle(/* private fields */);

Implementations

Source

impl IpcMemoryHandle

pub const unsafe fn from_raw(handle: cudaIpcMemHandle_t) -> Self

pub const fn zeroed() -> Self

pub const fn as_raw(&self) -> cudaIpcMemHandle_t

pub fn create_mapping<T>( self, flags: IpcMemoryFlags, ) -> Result<OpenedIpcMemory<T>>

Maps memory exported from another process with DeviceMemory::ipc_handle into the current device address space. For contexts on different devices, IpcMemoryHandle::create_mapping can attempt to enable peer access between the devices as if Device::enable_peer_access had been called. This behavior is controlled by IpcMemoryFlags::LAZY_ENABLE_PEER_ACCESS. Device::can_access_peer can determine if a mapping is possible.

IpcMemoryHandle::create_mapping can open handles to devices that may not be visible in the current process.

Imported memory handles from each device in a given process may only be opened by one context per device per other process.

If the memory handle has already been opened by the current context, the reference count on the handle is incremented by 1 and the existing device pointer is returned.

Memory returned from IpcMemoryHandle::create_mapping must be freed with OpenedIpcMemory::close.

Calling DeviceMemory::free on an exported memory region before calling OpenedIpcMemory::close in the importing context results in undefined behavior.

IPC is restricted to devices with support for unified addressing on Linux and Windows operating systems. IPC on Windows is supported for compatibility, but is not recommended because it has a performance cost. Check device IPC support through the device properties exposed by this crate, for example DeviceProperties::ipc_event_supported.

Additional CUDA diagnostics:

Errors

Returns an error if CUDA Runtime cannot open the IPC handle, if the current device cannot access the allocation, or if CUDA returns a null mapped pointer.