pub unsafe trait GraphRecordable {
type Output;
// Required method
fn record(self, scope: &StreamCaptureScope<'_>) -> Result<Self::Output>;
}Expand description
Operation that may be recorded into a CUDA graph capture scope.
Safety
Implementors must only enqueue CUDA work that is valid during stream capture. Every pointer, handle, and side effect captured into the resulting graph must have its replay safety contract represented by the operation’s type and constructor.
Required Associated Types
Required Methods
fn record(self, scope: &StreamCaptureScope<'_>) -> Result<Self::Output>
Dyn Compatibility
This trait is dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".