DevicePtr

Struct DevicePtr 

pub struct DevicePtr(/* private fields */);

Implementations

Source

impl DevicePtr

pub const fn null() -> Self

pub const unsafe fn new(raw: *mut ()) -> Self

Wraps a raw CUDA device pointer value.

Safety

raw must be either null or a pointer value that is valid for the CUDA operation it is passed to. This wrapper does not prove allocation ownership, size, lifetime, context association, or access permissions.

pub const unsafe fn from_raw(raw: *mut ()) -> Self

Wraps a raw CUDA device pointer value.

Safety

raw must be either null or a pointer value that is valid for the CUDA operation it is passed to. This wrapper does not prove allocation ownership, size, lifetime, context association, or access permissions.

pub const fn as_raw(self) -> *mut ()

pub const fn as_ptr(self) -> *mut ()

pub const fn as_const_ptr(self) -> *const ()

pub const fn is_null(self) -> bool

pub const fn cast<T>(self) -> *mut T