pub unsafe trait DeviceRepr: Copy + 'static { }Expand description
A Rust type that can be represented as plain CUDA device memory.
Safety
Implementors must have a stable bit representation for device memory. Values may be copied byte-for-byte between host and device memory without running Rust destructors or relying on host-only pointer validity.
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".